import tkinter as tk
from tkinter import ttk

class CommonFrame:

    def __init__(self, frame):

        frame.grid(row=0,column=0, padx=3, pady=3, sticky="EW")

        
        self.graphType = tk.StringVar()
        self.graphType.set("Select Graph Type")

        self.selectedFileVar = tk.StringVar()


#column 0
#====================================================================
        self.fileButton = tk.Button(frame, text="Select File")
        self.fileButton.grid(row=0, column=0, sticky="W", padx=(10,0), pady=(10,0))

        self.graphOptions = ttk.Combobox(frame, textvariable=self.graphType, values=["Scatterplot", "Interval Plot"], state="readonly")
        self.graphOptions.grid(row=2, column=0, columnspan=2, padx=(10,0), pady=5, sticky="W")


#column 1
#====================================================================
        self.selectedFile = tk.Entry(frame, textvariable=self.selectedFileVar, state="readonly", width=39)
        self.selectedFile.grid(row=0, column=1, padx=(3,0), pady=(10,0))

        self.selectedFileScroll = tk.Scrollbar(frame, orient="horizontal", command=self.selectedFile.xview)
        self.selectedFileScroll.grid(row=1,column=1,sticky="ew", pady=(0,10))

        self.selectedFile.config(xscrollcommand=self.selectedFileScroll.set)


import calculation
import rw
import numpy as np
import collections, re

d = {"key1" : 'object', "key11" : 'object', "key2" : 'object', "key22" : 'object', "jay1" : 'object', "jay2" : 'object'}


my_fun = lambda k,v: [k, int(v)]

string = "key11"

print(re.match(r'([a-zA-Z]+)(\d+)', string))

d2 = collections.OrderedDict(sorted(d.items(), key=lambda t: my_fun(*re.match(r'([a-zA-Z]+)(\d+)',t[0]).groups())))

print(d2)


import tkinter as tk
from tkinter import messagebox
from tkinter import ttk
from tkinter import filedialog
from pathlib import Path

from . import rw
from . import calculation
from . import graphing

class ScatterFrame:

    def __init__(self, frame):

        self.frame = frame

        self.xVarSelection = tk.StringVar()
        self.yVarSelection = tk.StringVar()
        self.cVarSelection = tk.StringVar()

        self.xGridCheckVal = tk.StringVar()
        self.yGridCheckVal = tk.StringVar()
        

#Column 0
#====================================================================

        frame.grid_columnconfigure(0, pad=25)

        self.xVarLabel = tk.Label(frame, text="X-Axis Column")
        self.xVarLabel.grid(row=0, column=0)

        self.xVar = ttk.Combobox(frame, textvariable=self.xVarSelection, values=[], state="readonly")
        self.xVar.grid(row=1, column=0)

        self.yVarLabel = tk.Label(frame, text="Y-Axis Column")
        self.yVarLabel.grid(row=2, column=0)

        self.yVar = ttk.Combobox(frame, textvariable=self.yVarSelection, values=[], state="readonly")
        self.yVar.grid(row=3, column=0)

        self.cVarLabel = tk.Label(frame, text="Category Column")
        self.cVarLabel.grid(row=4, column=0)

        self.cVar = ttk.Combobox(frame, textvariable=self.cVarSelection, values=[], state="readonly")
        self.cVar.grid(row=5, column=0)

        self.xGridCheck = tk.Checkbutton(frame, variable=self.xGridCheckVal, onvalue="x", offvalue="", text="X-axis grid lines")
        self.xGridCheck.grid(row=6, column=0)

        self.yGridCheck = tk.Checkbutton(frame, variable=self.yGridCheckVal, onvalue="y", offvalue="", text="Y-axis grid lines")
        self.yGridCheck.grid(row=7, column=0)

        

#Column 1
#====================================================================

        
        frame.grid_columnconfigure(1, pad=25)

        self.titleText = tk.Label(frame, text="Graph Title")
        self.titleText.grid(row=0, column=1)

        self.title = tk.Entry(frame, width=23)
        self.title.grid(row=1, column=1)

        self.xLabelText = tk.Label(frame, text="X-Axis Label")
        self.xLabelText.grid(row=2, column=1)

        self.xLabel = tk.Entry(frame, width=23)
        self.xLabel.grid(row=3, column=1)

        self.yLabelText = tk.Label(frame, text="Y-Axis Label")
        self.yLabelText.grid(row=4, column=1)

        self.yLabel = tk.Entry(frame, width=23)
        self.yLabel.grid(row=5, column=1)

        self.graphButton = tk.Button(frame, text="Create Scatterplot", command=self.createScatter)
        self.graphButton.grid(row=6, column=1, pady=(10,1))

        self.saveButton = tk.Button(frame, text=" Save Scatterplot ", command=self.saveScatter)
        self.saveButton.grid(row=7, column=1)

        

#====================================================================
#============================Functions===============================
#====================================================================

    def setFrame(self, columnLabels: list, data: list):

        self.columnLabels = columnLabels
        self.data = data

        self.xVar.config(values=columnLabels)
        self.yVar.config(values=columnLabels)
        self.cVar.config(values=["No Categories"]+columnLabels)

        self.xVarSelection.set("Select a Column")
        self.yVarSelection.set("Select a Column")
        self.cVarSelection.set("No Categories")

        self.xLabel.delete(0, tk.END)
        self.yLabel.delete(0, tk.END)
        self.title.delete(0, tk.END)

        self.xGridCheckVal.set("")
        self.yGridCheckVal.set("")

#event handler that uses the gui specified data to create a scatterplot
    def createScatter(self):

        if(self.xVar.current() == (-1) or self.yVar.current() == (-1)):
            messagebox.showinfo("Error", "Please select a column for the X-axis and Y-axis")
            self.graphButton.config(relief=tk.RAISED)
            return

        if(self.xLabel.get() == ""):
            self.xLabel.insert(0, self.columnLabels[self.xVar.current()])
        if(self.yLabel.get() == ""):
            self.yLabel.insert(0, self.columnLabels[self.yVar.current()])
        if(self.title.get() == ""):
            self.title.insert(0, self.columnLabels[self.xVar.current()] + " VS " + self.columnLabels[self.yVar.current()])

        groupColumn = [self.cVar.current()-1]
        if groupColumn == [-1]: groupColumn = None

        graphData = calculation.getColumns(data=self.data, xCol=self.xVar.current(),
                                           yCol=self.yVar.current(), groups=groupColumn)

        graphing.graphScatter(xs=graphData[0], ys=graphData[1], groups=graphData[2], xLabel=self.xLabel.get(),
                              yLabel=self.yLabel.get(), title=self.title.get(), gridLines=self.xGridCheckVal.get()+self.yGridCheckVal.get(), show = True)

    def saveScatter(self):
        if(self.xVar.current() == (-1) or self.yVar.current() == (-1)):
            messagebox.showinfo("Error", "Please select a column for the X-axis and Y-axis")
            self.graphButton.config(relief=tk.RAISED)
            return

        if(self.xLabel.get() == ""):
            self.xLabel.insert(0, self.columnLabels[self.xVar.current()])
        if(self.yLabel.get() == ""):
            self.yLabel.insert(0, self.columnLabels[self.yVar.current()])
        if(self.title.get() == ""):
            self.title.insert(0, self.columnLabels[self.xVar.current()] + " VS " + self.columnLabels[self.yVar.current()])

        groupColumn = [self.cVar.current()-1]
        if groupColumn == [-1]: groupColumn = None

        graphData = calculation.getColumns(data=self.data, xCol=self.xVar.current(),
                                           yCol=self.yVar.current(), groups=groupColumn)

        fileName = filedialog.asksaveasfilename(title = "Save File", defaultextension=".plot")

        if fileName != "":
            rw.saveScatter(xs=graphData[0], ys=graphData[1], groups=graphData[2], xLabel=self.xLabel.get(),
                                yLabel=self.yLabel.get(), title=self.title.get(), gridLines=self.xGridCheckVal.get()+self.yGridCheckVal.get(), fileName=fileName)


import matplotlib.pyplot as plt
import itertools
from collections import defaultdict
import natsort

from . import rw

#function that creates and shows the scatterplot
def graphScatter(*, xs: list, ys: list, groups: list = None, title: str = None,
                  xLabel: str = None, yLabel: str = None, gridLines: str = "", show: bool = False):

    _, _ = plt.subplots()
    
    if(groups != None):
        xPoints = defaultdict(list)
        yPoints = defaultdict(list)
        groupSet = list(set(groups[0]))

        for i in range(0,len(groups[0])):
            xPoints[groups[0][i]].append(xs[i])
            yPoints[groups[0][i]].append(ys[i])

        for i in range(0, len(groupSet)):
            plt.scatter(xPoints[groupSet[i]], yPoints[groupSet[i]], label=groupSet[i])
    else:
        plt.scatter(xs, ys)

    if(groups != None):
        plt.legend(loc="best").set_draggable(True)

    plt.title(title)
    plt.xlabel(xLabel)
    plt.ylabel(yLabel)

    if len(gridLines)==1:
        pass
        plt.grid(which="major", axis=gridLines)
    elif gridLines == "xy":
        pass
        plt.grid(which="major", axis="both")

    plt.tight_layout()

    if show:
        plt.show()
    else:
        return plt.gcf()

#function that creates and shows the interval plot
def graphInterval(*,data: dict, title: str = None, xLabel: str = None,
                  yLabel: str = None, gridLines: str = "", groupNames: tuple=(), show: bool=False):

    _, ax = plt.subplots()

    if len(groupNames) > 1:
        groupNames = (":\n".join(reversed(groupNames)))+":"
        plt.text(-.015,-.02, s=groupNames, horizontalalignment="right", verticalalignment="top", transform=ax.transAxes)

    sortedData = natsort.realsorted(data.items(), key = lambda t: t[0])

    for key, value in sortedData:

        inverseKey = "\n".join(reversed(key.split("\n")))

        if key in data:

            if value[0] != None:
                plt.scatter(inverseKey, value[0], marker="_", color="blue")
            plt.scatter(inverseKey, value[1], marker=".", color="blue")
            if value[2] != None:
                plt.scatter(inverseKey, value[2], marker="_", color="blue")
            if value[0] != None and value[2] != None:
                plt.plot([inverseKey]*3, [value[0], value[1], value[2]], linewidth=.85, color="blue")

    plt.title(title)
    plt.xlabel(xLabel)
    plt.ylabel(yLabel)

    if len(gridLines)==1:
        plt.grid(which="major", axis=gridLines)
    elif gridLines == "xy":
        plt.grid(which="major", axis="both")

    plt.tight_layout()

    if show:
        plt.show()
    else:
        return plt.gcf()

def figureFromFile(fileName, show: bool=False):

    figType = rw.getFigType(fileName)

    if figType == "scatter":
        xs,ys,groups,title,xLabel,yLabel,gridLines = rw.loadFigure(fileName)
        return graphScatter(xs=xs, ys=ys, groups=groups, title=title, xLabel=xLabel, yLabel=yLabel, gridLines=gridLines, show=show)
    elif figType == "interval":
        data,title,xLabel,yLabel,gridLines, groupNames = rw.loadFigure(fileName)
        return graphInterval(data=data, title=title, xLabel=xLabel, yLabel=yLabel, gridLines=gridLines, groupNames=groupNames, show=show)





import tkinter as tk
from tkinter import filedialog
from tkinter import messagebox
import tkinter.scrolledtext as st
from pathlib import Path

from . import interfaceCommon
from . import interfaceScatter
from . import interfaceInterval
from . import rw
from . import calculation
from . import graphing


class RootWindow:

    def __init__(self, root):

#Root modifcations
#====================================================================
        self.root = root
        root.title("Data Visualizer")
        root.grid_columnconfigure(0, weight=1)
        #root.geometry("343x282")                       #!!!SET TO NEW DIMENSIONS!!!#

#Frames
#====================================================================
        self.commonFrame = tk.Frame(root, borderwidth=1, relief=tk.SUNKEN)
        self.commonFrame = interfaceCommon.CommonFrame(self.commonFrame)
        
        self.scatterFrame = tk.Frame(root, borderwidth=1, relief=tk.SUNKEN)
        self.scatterFrame = interfaceScatter.ScatterFrame(self.scatterFrame)

        self.intervalFrame = tk.Frame(root, borderwidth=1, relief=tk.SUNKEN)
        self.intervalFrame = interfaceInterval.IntervalFrame(self.intervalFrame)

#Event Bindings
#====================================================================
        self.commonFrame.fileButton.bind("<Button-1>", self.loadFile)
        self.commonFrame.graphOptions.bind("<<ComboboxSelected>>", self.graphOptionSelected)


#====================================================================
#============================Functions===============================
#====================================================================


#function to prompt file selection dialogue and load file
#====================================================================
    def loadFile(self, event):

        self.commonFrame.fileButton.config(relief=tk.SUNKEN)


        #fileName = "C:/Users/halo2_000/Desktop/StatsViz/sampleData/created.csv"
        fileName = filedialog.askopenfilename(title = "Select a file", filetypes = (("csv and plot files","*.csv *.plot"),))

        if Path(fileName).is_file():

            if fileName[len(fileName)-3:len(fileName)] == "csv":

                self.commonFrame.selectedFileVar.set(fileName)
                self.commonFrame.selectedFile.xview_moveto(1)

                self.data, inconsistentCell = rw.read(fileName)

                if inconsistentCell != None:
                    messagebox.showinfo("Warning", "The data at row "+str(inconsistentCell[0])+" is inconsistent with the rest of the data in column "
                                        +str(inconsistentCell[1])+". The dataset may not plot properly")

                self.columnLabels = calculation.popLabels(data=self.data)

                self.scatterFrame.setFrame(self.columnLabels, self.data)
                self.intervalFrame.setFrame(self.columnLabels, self.data)

            else:
                graphing.figureFromFile(fileName, show=True)


#function to show options for selected graph type
#====================================================================
    def graphOptionSelected(self, event):

        if( not Path(self.commonFrame.selectedFileVar.get()).is_file()):
            messagebox.showinfo("Error", "Please select a file first")
            self.commonFrame.graphType.set("Select Graph Type")
            return

        if self.commonFrame.graphType.get() == "Scatterplot":
            self.intervalFrame.frame.grid_remove()
            self.scatterFrame.frame.grid(row=2, column=0, padx=3, pady=3, sticky="EW")
        elif self.commonFrame.graphType.get() == "Interval Plot":
            self.scatterFrame.frame.grid_remove()
            self.intervalFrame.frame.grid(row=2, column=0, padx=3, pady=3, sticky="EW")


from collections import defaultdict
from scipy import stats
import numpy as np

#gets the columns from the data that's specified by the input
def getColumns(*, data: [], xCol: int = None, yCol: int, groups: list = None):

    xs = [] #values in the specified x axis column
    ys = [] #values in the specified y axis column
    groupData = [] #list of columns used for grouping/categorization

#appends empty lists for each group
    if(groups != None):
        for i in range(0,len(groups)):
            groupData.append([])
    
#appends all the x and y data to xs and ys and groupData
    for i in range(0, len(data)):
        if(xCol != None):
            xs.append(data[i][xCol])
        ys.append(data[i][yCol])

        if(groups != None):
            for j in range(0, len(groups)):
                groupData[j].append(data[i][groups[j]])

#if there is no x axis (eg when using interval plot) and if there is no groupData
#There values will be set to None
    if(xCol == None):
        xs = None
    if(groups == None):
        groupData = None

    return (xs, ys, groupData)

#function that removes the first row of the csv, and returns there values as a list
def popLabels(*, data: []):
    return data.pop(0)

#function that calculates the necessary intervals as well as the mean for the interval plot
def getIntervals(*, type: str, ys: list, groups: list = None):

    calculatedData = {} #dict with all categories concatenated as key, and (+interval,mean,-interval) as value

#if there was at least one column specified for grouping
    if groups != None:
        groupConcat = []
        groupVals = defaultdict(list)
        
#creates key values for calculatedData
        for i in range(len(groups[0])):
            string = []
            for j in range(len(groups)):
                if(j != len(groups)-1):
                    string.append(str(groups[j][i])+"\n")
                else:
                    string.append(str(groups[j][i]))

            groupConcat.append("".join(string))

        groupSet = list(set(groupConcat)) #the set of all key combos that appear in the data

#calculates the values for each key in calculatedData
        for i in range(len(groupSet)):
            for j in range(len(ys)):
                if groupConcat[j] == groupSet[i]:
                    groupVals[groupSet[i]].append(ys[j])

            mean = np.mean(groupVals[groupSet[i]])
            interval = 0

        #determines what interval to calculate based on the type of interval specified
        #there must be more than one value at the group column combo for there to be an interval
            if len(groupVals[groupSet[i]]) > 1:
                if type == "se":  
                    interval = stats.sem(groupVals[groupSet[i]])
                elif type == "sd":
                    interval = np.std(groupVals[groupSet[i]])
                elif type == "ci":
                    interval = stats.sem(groupVals[groupSet[i]]) * stats.t.ppf((1.95) / 2, len(groupVals[groupSet[i]])-1)
                calculatedData[groupSet[i]] = (mean+interval, mean, mean-interval)
            else:
                calculatedData[groupSet[i]] = (None, mean, None)

#when there is no category specified for the interval plot (there will just be one mean and interval plotted)
    else:
        mean = np.mean(ys)
        interval = 0

        #determines what interval to calculate based on the type of interval specified
        #there must be more than one value at the group column combo for there to be an interval
        if len(ys) > 1:
            if type == "se":  
                interval = stats.sem(ys)
            elif type == "sd":
                interval = np.std(ys)
            elif type == "ci":
                interval = stats.sem(ys) * stats.t.ppf((1.95) / 2, len(ys)-1)
            calculatedData["interval"] = (mean+interval, mean, mean-interval)
        else:
            calculatedData["interval"] = (None, mean, None)

    return calculatedData